#include <stdio.h>
#include <gtk/gtk.h>
#include <gtkdatabox.h>
#include <math.h>
/* Some versions of math.h have a PI problem... */
#ifndef PI
#define PI 3.14159265358979323846
#endif

#define POINTS 2000

/*----------------------------------------------------------------
 * data allocation
 *----------------------------------------------------------------*/
static void
data_alloc(guint **set_points, guint **type, 
	gfloat ***X, gfloat ***Y, gfloat ***Z,
	guint num_sets, guint set_points_used, gboolean different_x_y)
{
  gint i;
  
  *set_points=(guint *) g_malloc(num_sets*sizeof(guint));
  *type=(guint *) g_malloc(num_sets*sizeof(guint));
  *X=(gfloat **) g_malloc(num_sets*sizeof(gfloat *));
  *Y=(gfloat **) g_malloc(num_sets*sizeof(gfloat *));
  *Z=(gfloat **) g_malloc(num_sets*sizeof(gfloat *));

  for (i=0; i<num_sets; i++) {
    set_points[0][i]=POINTS;
    type[0][i]=GTK_DATABOX_POINTS_2D;
    if (!i || different_x_y) {
      X[0][i]=(gfloat *) g_malloc(set_points_used*sizeof(gfloat));
      Y[0][i]=(gfloat *) g_malloc(set_points_used*sizeof(gfloat));
    } else {
      X[0][i]=X[0][0];
      Y[0][i]=Y[0][0];
    }
    Z[0][i]=(gfloat *) g_malloc(set_points_used*sizeof(gfloat));

  }
}

/*----------------------------------------------------------------
 *  databox basics
 *----------------------------------------------------------------*/

static guint    basicNumSets=0;
static guint   *basicSetPoints=NULL;
static guint   *basicType=NULL;
static gfloat **basicX=NULL;
static gfloat **basicY=NULL;
static gfloat **basicZ=NULL;

static void
create_basics(void)
{
  static GtkWidget *window = NULL;
  GtkWidget *box1;
  GtkWidget *box2;
  GtkWidget *close_button;
  GtkWidget *box;
  GtkWidget *label;
  GtkWidget *separator;
  gint i;
  
  if (!window)
  {
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_usize (window, 300, 300);  

    gtk_signal_connect (GTK_OBJECT (window), "destroy",
			  GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			  &window);

    gtk_window_set_title (GTK_WINDOW (window), "Databox Basics");
    gtk_container_border_width (GTK_CONTAINER (window), 0);

    box1 = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), box1);

    label=gtk_label_new("Draw a selection with the left button pressed,\nThan click into the selection.\nUse the right mouse button to zoom out.\nShift+mouse button zooms to default.");
    gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
    separator=gtk_hseparator_new();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);
  
    box=gtk_databox_new ();
    if (!basicNumSets) {
      basicNumSets=1;
      data_alloc(&basicSetPoints, &basicType, &basicX, &basicY, &basicZ, basicNumSets, POINTS, FALSE);
      for (i = 0; i < POINTS; i++ ) {
        basicX[0][i]=i;
        basicY[0][i]=0;
        basicZ[0][i]=100.*sin(i*2*PI/POINTS);
      }
    }
    GTK_DATABOX(box)->data.num_sets=basicNumSets;
    GTK_DATABOX(box)->data.set_points=basicSetPoints;
    GTK_DATABOX(box)->data.type=basicType;
    GTK_DATABOX(box)->data.X=basicX;
    GTK_DATABOX(box)->data.Y=basicY;
    GTK_DATABOX(box)->data.Z=basicZ;
    gtk_databox_rescale(GTK_DATABOX(box));
    gtk_box_pack_start (GTK_BOX (box1), box, TRUE, TRUE, 0);

    separator = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_end (GTK_BOX (box1), box2, FALSE, TRUE, 0);
    close_button = gtk_button_new_with_label ("close");
    gtk_signal_connect_object (GTK_OBJECT (close_button), "clicked",
			 GTK_SIGNAL_FUNC(gtk_widget_destroy),
			 GTK_OBJECT (window));
    gtk_box_pack_start (GTK_BOX (box2), close_button, TRUE, TRUE, 0);
    GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (close_button);

  }
  
  if (!GTK_WIDGET_VISIBLE (window))
    gtk_widget_show_all (window);
  else {
    gtk_widget_destroy (window);
  }
}


/*----------------------------------------------------------------
 *  databox colors and misc
 *----------------------------------------------------------------*/

static guint    colorNumSets=0;
static guint   *colorSetPoints=NULL;
static guint   *colorType=NULL;
static gfloat **colorX=NULL;
static gfloat **colorY=NULL;
static gfloat **colorZ=NULL;

static void menu_color_change_cb		(gpointer callback_data, guint callback_action, GtkWidget *widget);
static void menu_options_show_coord_cb		(gpointer callback_data, guint callback_action, GtkWidget *widget);

static GtkItemFactoryEntry
menu_main_entries[]=
{
  {"/_Color", 				NULL, NULL, 		0, 	"<Branch>"},
  {"/Color/Change Color #0",		NULL, menu_color_change_cb, 	0, 	NULL},
  {"/Color/Change Color #1",		NULL, menu_color_change_cb,	1, 	NULL},
  {"/Color/Change Color #2",		NULL, menu_color_change_cb,	2, 	NULL},
  {"/Color/Change Color #3",		NULL, menu_color_change_cb,	3, 	NULL},
  {"/Color/Change Color #4",		NULL, menu_color_change_cb,	4, 	NULL},
  {"/_Options", 			NULL, NULL, 		0, 	"<Branch>"},
  {"/Options/Hide coordinates",		NULL, menu_options_show_coord_cb,	0, 	"<RadioItem>"},
  {"/Options/Show coordinates",		NULL, menu_options_show_coord_cb,	1, 	"/Options/Hide coordinates"},
  {"/_Help", 				NULL, NULL, 				0, 	"<LastBranch>"},
  {"/Help/Do not hesitate to ask\nrbock@eudoxos.de", 	NULL, NULL, 		0, 	NULL},
};

static gint
menu_main_num = sizeof (menu_main_entries) / sizeof (menu_main_entries[0]);

typedef struct {
  GtkWidget *selector;
  GtkWidget *box;
  gint index;
} col_sel;

static void 
get_color_cb(GtkWidget *widget, col_sel *sel)
{
  gdouble color[8];
  
  g_return_if_fail(GTK_IS_COLOR_SELECTION_DIALOG(sel->selector));
  g_return_if_fail(GTK_IS_DATABOX(sel->box));

  gtk_color_selection_get_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(sel->selector)->colorsel), color);
  gtk_databox_set_color(GTK_DATABOX(sel->box), sel->index, (gushort)(color[0]*65535), (gushort)(color[1]*65535), (gushort)(color[2]*65535));
  gtk_databox_redraw(GTK_DATABOX(sel->box));
}

static void
menu_color_change_cb(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  GtkWidget *selector;
  gchar title[20];
  col_sel *sel=(col_sel *) g_malloc(sizeof(col_sel));
  gushort red, green, blue;
  gdouble color[8];
  
  sprintf(title, "Choose color #%d", callback_action);
  selector=gtk_color_selection_dialog_new(title);
  gtk_widget_destroy(GTK_COLOR_SELECTION_DIALOG(selector)->help_button);
  sel->selector=selector;
  sel->box=GTK_WIDGET(callback_data);
  sel->index=callback_action;
  
  gtk_databox_get_color(GTK_DATABOX(sel->box), sel->index, &red, &green, &blue);
  color[0]=red/65535.;
  color[1]=green/65535.;
  color[2]=blue/65535.;
  color[3]=1;		/* Opacity? Dunno how to use it. */
  gtk_color_selection_set_color(GTK_COLOR_SELECTION(GTK_COLOR_SELECTION_DIALOG(sel->selector)->colorsel), color);
  
  gtk_signal_connect_object(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(selector)->cancel_button), "clicked",
		 GTK_SIGNAL_FUNC(gtk_widget_destroy),
		 GTK_OBJECT(selector));
  gtk_signal_connect(GTK_OBJECT(GTK_COLOR_SELECTION_DIALOG(selector)->ok_button), "clicked",
		 GTK_SIGNAL_FUNC(get_color_cb),
		 (gpointer) sel);
  gtk_widget_show(selector);

  return;
}

static void menu_options_show_coord_cb		(gpointer callback_data, guint callback_action, GtkWidget *widget)
{
  gtk_databox_show_cross(GTK_DATABOX(callback_data), callback_action);
  gtk_databox_redraw(GTK_DATABOX(callback_data));
}

static void
create_colors(void)
{
  static GtkWidget *window = NULL;
  GtkWidget *box1=NULL;
  GtkWidget *box2=NULL;
  GtkWidget *close_button=NULL;
  GtkWidget *box=NULL;
  GtkWidget *separator;
  GtkWidget *label;
  GtkWidget *menu_bar;
  GtkItemFactory *item_factory;
  gint i;
  gchar title[]="<Main>";
  
  if (!window)
  {
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_usize (window, 300, 300);  

    gtk_signal_connect (GTK_OBJECT (window), "destroy",
			  GTK_SIGNAL_FUNC(gtk_widget_destroyed),
			  &window);

    gtk_window_set_title (GTK_WINDOW (window), "Databox Basics");
    gtk_container_border_width (GTK_CONTAINER (window), 0);

    box1 = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), box1);

    box=gtk_databox_new ();
    item_factory=gtk_item_factory_new(GTK_TYPE_MENU_BAR, title, NULL);
    gtk_item_factory_create_items(item_factory, menu_main_num, menu_main_entries, box);
    menu_bar=gtk_item_factory_get_widget(item_factory, title);
    gtk_box_pack_start(GTK_BOX (box1), menu_bar, FALSE, TRUE, 0);

    label=gtk_label_new("You may change the colors of the shown datasets,\nthe drawing style and behaviour of\ncoordinate axes and/or scrollbars.");
    gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
    separator=gtk_hseparator_new();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);
  
    if (!colorNumSets) {
      colorNumSets=5;
      data_alloc(&colorSetPoints, &colorType, &colorX, &colorY, &colorZ, colorNumSets, POINTS, FALSE);
    for (i = 0; i < POINTS; i++ ) {
       colorX[0][i]=i;
       colorY[0][i]=0;
       colorZ[0][i]=100.*sin(i*2*PI/POINTS);
       colorZ[1][i]=100.*sin(i*4*PI/POINTS);
       colorZ[2][i]=100.*sin(i*6*PI/POINTS);
       colorZ[3][i]=100.*sin(i*8*PI/POINTS);
       colorZ[4][i]=100.*sin(i*10*PI/POINTS);
    }
    GTK_DATABOX(box)->data.num_sets=colorNumSets;
    GTK_DATABOX(box)->data.set_points=colorSetPoints;
    GTK_DATABOX(box)->data.type=colorType;
    GTK_DATABOX(box)->data.X=colorX;
    GTK_DATABOX(box)->data.Y=colorY;
    GTK_DATABOX(box)->data.Z=colorZ;
    }
    gtk_databox_rescale(GTK_DATABOX(box));
    gtk_box_pack_start (GTK_BOX (box1), box, TRUE, TRUE, 0);

    separator = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_end (GTK_BOX (box1), box2, FALSE, TRUE, 0);
    close_button = gtk_button_new_with_label ("close");
    gtk_signal_connect_object (GTK_OBJECT (close_button), "clicked",
			 GTK_SIGNAL_FUNC(gtk_widget_destroy),
			 GTK_OBJECT (window));
    gtk_box_pack_start (GTK_BOX (box2), close_button, TRUE, TRUE, 0);
    GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (close_button);
  }
  
  if (!GTK_WIDGET_VISIBLE (window))
    gtk_widget_show_all (window);
  else {
    gtk_widget_destroy (window);
  }
}

/*----------------------------------------------------------------
 *  databox lissajous
 *----------------------------------------------------------------*/

static guint    lissajousNumSets=0;
static guint   *lissajousSetPoints=NULL;
static guint   *lissajousType=NULL;
static gfloat **lissajousX=NULL;
static gfloat **lissajousY=NULL;
static gfloat **lissajousZ=NULL;

static gint     lissajous_idle=0;
static gfloat   lissajous_frequency=3.*PI/2.;
static GtkWidget *lissajous_label=NULL;
static guint    lissajous_counter=0;

static gboolean
lissajous_idle_func(GtkDatabox *box)
{
  gfloat freq;
  gchar label[10];
  gint i;

  if (!GTK_IS_DATABOX(box)) return FALSE;

  lissajous_frequency+=0.001;
  
  freq=14+10*sin(lissajous_frequency);
  for (i = 0; i < POINTS; i++ ) {
     GTK_DATABOX(box)->data.X[0][i]=100.*sin(i*4*PI/POINTS);
     GTK_DATABOX(box)->data.Y[0][i]=0;
     GTK_DATABOX(box)->data.Z[0][i]=100.*cos(i*freq*PI/POINTS);
  }
  
  gtk_databox_redraw(GTK_DATABOX(box));
  
  sprintf(label,"%d", lissajous_counter++);
  gtk_entry_set_text(GTK_ENTRY(lissajous_label), label);
  
  return TRUE;  
} 

static void
lissajous_destroy(GtkWidget *widget, GtkWidget **window)
{
  if (lissajous_idle) {
    gtk_idle_remove(lissajous_idle);
    lissajous_idle=0;
  }
  *window=NULL;  
}

static void
create_lissajous(void)
{
  static GtkWidget *window = NULL;
  GtkWidget *box1;
  GtkWidget *box2;
  GtkWidget *close_button;
  GtkWidget *box;
  GtkWidget *label;
  GtkWidget *separator;
  gint i;
  
  if (!window)
  {
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_usize (window, 300, 300);  

    gtk_signal_connect (GTK_OBJECT (window), "destroy",
                          GTK_SIGNAL_FUNC(lissajous_destroy),
                          &window);

    gtk_window_set_title (GTK_WINDOW (window), "Databox Basics");
    gtk_container_border_width (GTK_CONTAINER (window), 0);

    box1 = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), box1);

    label=gtk_label_new("This example resembles an oszilloscope\nreceiving two signals, one is a sine (horizontal), \nthe other is a cosine with variable frequency(vertical).\nThe counter is synchron with the updates.");
    gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
    separator=gtk_hseparator_new();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);
    lissajous_label=gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(lissajous_label), "0");
    gtk_box_pack_start (GTK_BOX (box1), lissajous_label, FALSE, FALSE, 0);
    separator=gtk_hseparator_new();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);

    lissajous_idle=0;
    lissajous_frequency=3.*PI/2.;
    lissajous_counter=0;

    box=gtk_databox_new ();
    if (!lissajousNumSets) {
      lissajousNumSets=1;
      data_alloc(&lissajousSetPoints, &lissajousType, &lissajousX, &lissajousY, &lissajousZ, lissajousNumSets, POINTS, FALSE);
      for (i = 0; i < POINTS; i++ ) {
        lissajousX[0][i]=100.*sin(i*4*PI/POINTS);
        lissajousY[0][i]=0;
        lissajousZ[0][i]=100.*cos(i*4*PI/POINTS);
      }
    }
    GTK_DATABOX(box)->data.num_sets=lissajousNumSets;
    GTK_DATABOX(box)->data.set_points=lissajousSetPoints;
    GTK_DATABOX(box)->data.type=lissajousType;
    GTK_DATABOX(box)->data.X=lissajousX;
    GTK_DATABOX(box)->data.Y=lissajousY;
    GTK_DATABOX(box)->data.Z=lissajousZ;
    gtk_databox_rescale(GTK_DATABOX(box));
    gtk_box_pack_start (GTK_BOX (box1), box, TRUE, TRUE, 0);

    separator = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_end (GTK_BOX (box1), box2, FALSE, TRUE, 0);
    close_button = gtk_button_new_with_label ("close");

    gtk_signal_connect_object_after (GTK_OBJECT (close_button), "clicked",
			 GTK_SIGNAL_FUNC(gtk_widget_destroy),
			 GTK_OBJECT (window));

    gtk_box_pack_start (GTK_BOX (box2), close_button, TRUE, TRUE, 0);
    GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (close_button);
    lissajous_idle=gtk_idle_add((GtkFunction) lissajous_idle_func, box);
  }
  
  if (!GTK_WIDGET_VISIBLE (window))
    gtk_widget_show_all (window);
  else {
    gtk_widget_destroy (window);
  }
}

/*----------------------------------------------------------------
 *  databox movement
 *----------------------------------------------------------------*/

static guint    movementNumSets=0;
static guint   *movementSetPoints=NULL;
static guint   *movementType=NULL;
static gfloat **movementX=NULL;
static gfloat **movementY=NULL;
static gfloat **movementZ=NULL;

static gint     movement_idle=0;
static gfloat   movement_length=100;
static GtkWidget *movement_label=NULL;
static guint    movement_counter=0;

static gboolean
movement_idle_func(GtkDatabox *box)
{
  gchar label[10];
  gint i;

  if (!GTK_IS_DATABOX(box)) return FALSE;

  for (i = 0; i < movement_length; i++ ) {
  GTK_DATABOX(box)->data.X[1][i]=100.*sin((movement_counter+i)*2*PI/POINTS)+2*sin(i*2*PI/20);
  GTK_DATABOX(box)->data.Y[1][i]=0;
  GTK_DATABOX(box)->data.Z[1][i]=100.*cos((movement_counter+i)*10*PI/POINTS)+2*cos(i*2*PI/20);
  }
  
  gtk_databox_redraw(GTK_DATABOX(box));
  
  sprintf(label,"%d", movement_counter++);
  gtk_entry_set_text(GTK_ENTRY(movement_label), label);
  
  return TRUE;  
} 

static void
movement_destroy(GtkWidget *widget, GtkWidget **window)
{
  if (movement_idle) {
    gtk_idle_remove(movement_idle);
    movement_idle=0;
  }
  *window=NULL;  
}

static void
create_movement(void)
{
  static GtkWidget *window = NULL;
  GtkWidget *box1;
  GtkWidget *box2;
  GtkWidget *close_button;
  GtkWidget *box;
  GtkWidget *label;
  GtkWidget *separator;
  gint i;
  
  if (!window)
  {
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_widget_set_usize (window, 300, 300);  

    gtk_signal_connect (GTK_OBJECT (window), "destroy",
                          GTK_SIGNAL_FUNC(movement_destroy),
                          &window);

    gtk_window_set_title (GTK_WINDOW (window), "Databox Basics");
    gtk_container_border_width (GTK_CONTAINER (window), 0);

    box1 = gtk_vbox_new (FALSE, 0);
    gtk_container_add (GTK_CONTAINER (window), box1);

    label=gtk_label_new("This example resembles an oszilloscope\nreceiving two signals, one is a sine (horizontal), \nthe other is a cosine with variable frequency(vertical).\nThe counter is synchron with the updates.");
    gtk_box_pack_start (GTK_BOX (box1), label, FALSE, FALSE, 0);
    separator=gtk_hseparator_new();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);
    movement_label=gtk_entry_new();
    gtk_entry_set_text(GTK_ENTRY(movement_label), "0");
    gtk_box_pack_start (GTK_BOX (box1), movement_label, FALSE, FALSE, 0);
    separator=gtk_hseparator_new();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, FALSE, 0);

    movement_idle=0;
    movement_counter=0;

    box=gtk_databox_new ();
    if (!movementNumSets) {
      movementNumSets=2;
      data_alloc(&movementSetPoints, &movementType, &movementX, &movementY, &movementZ, movementNumSets, POINTS+movement_length, FALSE);
      movementX[1]=(gfloat *) g_malloc(movement_length*sizeof(float));
      movementY[1]=(gfloat *) g_malloc(movement_length*sizeof(float));
      movementZ[1]=(gfloat *) g_malloc(movement_length*sizeof(float));

      movementSetPoints[1]=movement_length;

      for (i = 0; i < POINTS+movement_length; i++ ) {
        movementX[0][i]=100.*sin(i*2*PI/POINTS);
        movementY[0][i]=0;
        movementZ[0][i]=100.*cos(i*10*PI/POINTS);
        if (i<movement_length) {
          movementX[1][i]=100.*sin(i*2*PI/POINTS);
          movementY[1][i]=0;
          movementZ[1][i]=100.*cos(i*10*PI/POINTS);
        }
      }
    }
    GTK_DATABOX(box)->data.num_sets=movementNumSets;
    GTK_DATABOX(box)->data.set_points=movementSetPoints;
    GTK_DATABOX(box)->data.type=movementType;
    GTK_DATABOX(box)->data.X=movementX;
    GTK_DATABOX(box)->data.Y=movementY;
    GTK_DATABOX(box)->data.Z=movementZ;
    gtk_databox_rescale(GTK_DATABOX(box));
    gtk_databox_set_color(GTK_DATABOX(box), 1, 0, 65535, 0);
    gtk_box_pack_start (GTK_BOX (box1), box, TRUE, TRUE, 0);

    separator = gtk_hseparator_new ();
    gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

    box2 = gtk_vbox_new (FALSE, 10);
    gtk_container_border_width (GTK_CONTAINER (box2), 10);
    gtk_box_pack_end (GTK_BOX (box1), box2, FALSE, TRUE, 0);
    close_button = gtk_button_new_with_label ("close");

    gtk_signal_connect_object_after (GTK_OBJECT (close_button), "clicked",
			 GTK_SIGNAL_FUNC(gtk_widget_destroy),
			 GTK_OBJECT (window));

    gtk_box_pack_start (GTK_BOX (box2), close_button, TRUE, TRUE, 0);
    GTK_WIDGET_SET_FLAGS (close_button, GTK_CAN_DEFAULT);
    gtk_widget_grab_default (close_button);
    movement_idle=gtk_idle_add((GtkFunction) movement_idle_func, box);
  }
  
  if (!GTK_WIDGET_VISIBLE (window))
    gtk_widget_show_all (window);
  else {
    gtk_widget_destroy (window);
  }
}

static void
do_exit (GtkWidget *widget, GtkWidget *window)
{
  gtk_widget_destroy (window);
  gtk_main_quit ();
}

static void
create_main_window (void)
{
  struct {
    char *label;
    void (*func) ();
  } buttons[] =
    {
      { "databox basics", create_basics },
      { "databox colors and misc", create_colors },
      { "lissajous example", create_lissajous },
      { "movement example", create_movement },
    };
  int nbuttons = sizeof (buttons) / sizeof (buttons[0]);
  GtkWidget *window;
  GtkWidget *box1;
  GtkWidget *box2;
  GtkWidget *button;
  GtkWidget *separator;
  int i;

  window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_window_set_policy (GTK_WINDOW (window), FALSE, FALSE, FALSE);
  gtk_widget_set_name (window, "Gtk Databox");
  gtk_widget_set_usize (window, 200, 200);
  gtk_widget_set_uposition (window, 20, 20);

  gtk_signal_connect (GTK_OBJECT (window), "destroy",
		      GTK_SIGNAL_FUNC(gtk_main_quit),
		      NULL);
  gtk_signal_connect (GTK_OBJECT (window), "delete-event",
		      GTK_SIGNAL_FUNC (gtk_false),
		      NULL);

  box1 = gtk_vbox_new (FALSE, 0);
  gtk_container_add (GTK_CONTAINER (window), box1);

  gtk_container_border_width (GTK_CONTAINER (box1), 10);

  for (i = 0; i < nbuttons; i++)
    {
      button = gtk_button_new_with_label (buttons[i].label);
      if (buttons[i].func)
        gtk_signal_connect (GTK_OBJECT (button), 
			    "clicked", 
			    GTK_SIGNAL_FUNC(buttons[i].func),
			    NULL);
      else
        gtk_widget_set_sensitive (button, FALSE);
      gtk_box_pack_start (GTK_BOX (box1), button, TRUE, TRUE, 0);
    }

  separator = gtk_hseparator_new ();
  gtk_box_pack_start (GTK_BOX (box1), separator, FALSE, TRUE, 0);

  box2 = gtk_vbox_new (FALSE, 10);
  gtk_container_border_width (GTK_CONTAINER (box2), 10);
  gtk_box_pack_start (GTK_BOX (box1), box2, FALSE, TRUE, 0);

  button = gtk_button_new_with_label ("close");
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
		      GTK_SIGNAL_FUNC (do_exit),
		      window);
  gtk_box_pack_start (GTK_BOX (box2), button, TRUE, TRUE, 0);
  GTK_WIDGET_SET_FLAGS (button, GTK_CAN_DEFAULT);
  gtk_widget_grab_default (button);

  gtk_widget_show_all (window);
}

gint 
main(gint argc, char *argv[])
{
  gtk_init(&argc, &argv);
  
  create_main_window();
  
  gtk_main();

  return 0;
}
